package javax.swing.table;

import java.awt.event.FocusListener;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Font;
import java.awt.Cursor;
import java.awt.Color;
import javax.accessibility.AccessibleValue;
import javax.accessibility.AccessibleText;
import javax.accessibility.AccessibleSelection;
import javax.accessibility.AccessibleAction;
import java.beans.PropertyChangeListener;
import java.util.Locale;
import javax.accessibility.AccessibleState;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleComponent;
import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectOutputStream;
import javax.swing.ToolTipManager;
import sun.swing.table.DefaultTableCellHeaderRenderer;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ChangeEvent;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.plaf.TableHeaderUI;
import java.awt.Component;
import sun.awt.AWTAccessor;
import java.awt.event.MouseEvent;
import java.awt.Rectangle;
import java.awt.Point;
import java.beans.Transient;
import javax.swing.JTable;
import javax.accessibility.Accessible;
import javax.swing.event.TableColumnModelListener;
import javax.swing.JComponent;

public class JTableHeader extends JComponent implements TableColumnModelListener, Accessible
{
    private static final String uiClassID = "TableHeaderUI";
    protected JTable table;
    protected TableColumnModel columnModel;
    protected boolean reorderingAllowed;
    protected boolean resizingAllowed;
    protected boolean updateTableInRealTime;
    protected transient TableColumn resizingColumn;
    protected transient TableColumn draggedColumn;
    protected transient int draggedDistance;
    private TableCellRenderer defaultRenderer;
    
    public JTableHeader() {
        this(null);
    }
    
    public JTableHeader(TableColumnModel defaultColumnModel) {
        if (defaultColumnModel == null) {
            defaultColumnModel = this.createDefaultColumnModel();
        }
        this.setColumnModel(defaultColumnModel);
        this.initializeLocalVars();
        this.updateUI();
    }
    
    public void setTable(JTable table) {
        this.firePropertyChange("table", this.table, this.table = table);
    }
    
    public JTable getTable() {
        return this.table;
    }
    
    public void setReorderingAllowed(boolean reorderingAllowed) {
        this.firePropertyChange("reorderingAllowed", this.reorderingAllowed, this.reorderingAllowed = reorderingAllowed);
    }
    
    public boolean getReorderingAllowed() {
        return this.reorderingAllowed;
    }
    
    public void setResizingAllowed(boolean resizingAllowed) {
        this.firePropertyChange("resizingAllowed", this.resizingAllowed, this.resizingAllowed = resizingAllowed);
    }
    
    public boolean getResizingAllowed() {
        return this.resizingAllowed;
    }
    
    public TableColumn getDraggedColumn() {
        return this.draggedColumn;
    }
    
    public int getDraggedDistance() {
        return this.draggedDistance;
    }
    
    public TableColumn getResizingColumn() {
        return this.resizingColumn;
    }
    
    public void setUpdateTableInRealTime(boolean updateTableInRealTime) {
        this.updateTableInRealTime = updateTableInRealTime;
    }
    
    public boolean getUpdateTableInRealTime() {
        return this.updateTableInRealTime;
    }
    
    public void setDefaultRenderer(TableCellRenderer defaultRenderer) {
        this.defaultRenderer = defaultRenderer;
    }
    
    @Transient
    public TableCellRenderer getDefaultRenderer() {
        return this.defaultRenderer;
    }
    
    public int columnAtPoint(Point point) {
        int x = point.x;
        if (!this.getComponentOrientation().isLeftToRight()) {
            x = this.getWidthInRightToLeft() - x - 1;
        }
        return this.getColumnModel().getColumnIndexAtX(x);
    }
    
    public Rectangle getHeaderRect(int n) {
        Rectangle rectangle = new Rectangle();
        TableColumnModel columnModel = this.getColumnModel();
        rectangle.height = this.getHeight();
        if (n < 0) {
            if (!this.getComponentOrientation().isLeftToRight()) {
                rectangle.x = this.getWidthInRightToLeft();
            }
        }
        else if (n >= columnModel.getColumnCount()) {
            if (this.getComponentOrientation().isLeftToRight()) {
                rectangle.x = this.getWidth();
            }
        }
        else {
            for (int i = 0; i < n; ++i) {
                rectangle.x += columnModel.getColumn(i).getWidth();
            }
            if (!this.getComponentOrientation().isLeftToRight()) {
                rectangle.x = this.getWidthInRightToLeft() - rectangle.x - columnModel.getColumn(n).getWidth();
            }
            rectangle.width = columnModel.getColumn(n).getWidth();
        }
        return rectangle;
    }
    
    @Override
    public String getToolTipText(MouseEvent mouseEvent) {
        String s = null;
        Point point = mouseEvent.getPoint();
        int columnAtPoint;
        if ((columnAtPoint = this.columnAtPoint(point)) != -1) {
            TableColumn column = this.columnModel.getColumn(columnAtPoint);
            TableCellRenderer tableCellRenderer = column.getHeaderRenderer();
            if (tableCellRenderer == null) {
                tableCellRenderer = this.defaultRenderer;
            }
            Component tableCellRendererComponent = tableCellRenderer.getTableCellRendererComponent(this.getTable(), column.getHeaderValue(), false, false, -1, columnAtPoint);
            if ((tableCellRendererComponent instanceof JComponent)) {
                Rectangle headerRect = this.getHeaderRect(columnAtPoint);
                point.translate(-headerRect.x, -headerRect.y);
                MouseEvent event = new MouseEvent(tableCellRendererComponent, mouseEvent.getID(), mouseEvent.getWhen(), mouseEvent.getModifiers(), point.x, point.y, mouseEvent.getXOnScreen(), mouseEvent.getYOnScreen(), mouseEvent.getClickCount(), mouseEvent.isPopupTrigger(), 0);
                AWTAccessor.MouseEventAccessor mouseEventAccessor = AWTAccessor.getMouseEventAccessor();
                mouseEventAccessor.setCausedByTouchEvent(event, mouseEventAccessor.isCausedByTouchEvent(mouseEvent));
                s = ((JComponent)tableCellRendererComponent).getToolTipText(event);
            }
        }
        if (s == null) {
            s = super.getToolTipText();
            return s;
        }
        return s;
    }
    
    @Override
    public TableHeaderUI getUI() {
        return (TableHeaderUI)this.ui;
    }
    
    public void setUI(TableHeaderUI ui) {
        if (this.ui != ui) {
            super.setUI(ui);
            this.repaint();
        }
    }
    
    @Override
    public void updateUI() {
        this.setUI((TableHeaderUI)UIManager.getUI(this));
        TableCellRenderer defaultRenderer = this.getDefaultRenderer();
        if ((defaultRenderer instanceof Component)) {
            SwingUtilities.updateComponentTreeUI((Component)defaultRenderer);
        }
    }
    
    @Override
    public String getUIClassID() {
        return "TableHeaderUI";
    }
    
    public void setColumnModel(TableColumnModel tableColumnModel) {
        if (tableColumnModel == null) {
            throw new IllegalArgumentException("Cannot set a null ColumnModel");
        }
        TableColumnModel columnModel = this.columnModel;
        if (tableColumnModel != columnModel) {
            if (columnModel != null) {
                columnModel.removeColumnModelListener(this);
            }
            (this.columnModel = tableColumnModel).addColumnModelListener(this);
            this.firePropertyChange("columnModel", columnModel, tableColumnModel);
            this.resizeAndRepaint();
        }
        return;
    }
    
    public TableColumnModel getColumnModel() {
        return this.columnModel;
    }
    
    @Override
    public void columnAdded(TableColumnModelEvent tableColumnModelEvent) {
        this.resizeAndRepaint();
    }
    
    @Override
    public void columnRemoved(TableColumnModelEvent tableColumnModelEvent) {
        this.resizeAndRepaint();
    }
    
    @Override
    public void columnMoved(TableColumnModelEvent tableColumnModelEvent) {
        this.repaint();
    }
    
    @Override
    public void columnMarginChanged(ChangeEvent changeEvent) {
        this.resizeAndRepaint();
    }
    
    @Override
    public void columnSelectionChanged(ListSelectionEvent listSelectionEvent) {
    }
    
    protected TableColumnModel createDefaultColumnModel() {
        return new DefaultTableColumnModel();
    }
    
    protected TableCellRenderer createDefaultRenderer() {
        return new DefaultTableCellHeaderRenderer();
    }
    
    protected void initializeLocalVars() {
        this.setOpaque(true);
        this.table = null;
        this.reorderingAllowed = true;
        this.resizingAllowed = true;
        this.draggedColumn = null;
        this.draggedDistance = 0;
        this.resizingColumn = null;
        this.updateTableInRealTime = true;
        ToolTipManager.sharedInstance().registerComponent(this);
        this.setDefaultRenderer(this.createDefaultRenderer());
    }
    
    public void resizeAndRepaint() {
        this.revalidate();
        this.repaint();
    }
    
    public void setDraggedColumn(TableColumn draggedColumn) {
        this.draggedColumn = draggedColumn;
    }
    
    public void setDraggedDistance(int draggedDistance) {
        this.draggedDistance = draggedDistance;
    }
    
    public void setResizingColumn(TableColumn resizingColumn) {
        this.resizingColumn = resizingColumn;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.ui != null && this.getUIClassID().equals("TableHeaderUI")) {
            this.ui.installUI(this);
        }
    }
    
    private int getWidthInRightToLeft() {
        if (this.table != null && this.table.getAutoResizeMode() != 0) {
            return this.table.getWidth();
        }
        return super.getWidth();
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",draggedDistance=" + this.draggedDistance + ",reorderingAllowed=" + (this.reorderingAllowed ? "true" : "false") + ",resizingAllowed=" + (this.resizingAllowed ? "true" : "false") + ",updateTableInRealTime=" + (this.updateTableInRealTime ? "true" : "false");
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JTableHeader.AccessibleJTableHeader();
        }
        return this.accessibleContext;
    }
    
    protected class AccessibleJTableHeader extends JComponent.AccessibleJComponent
    {
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.PANEL;
        }
        
        @Override
        public Accessible getAccessibleAt(Point point) {
            int columnAtPoint;
            if ((columnAtPoint = JTableHeader.this.columnAtPoint(point)) != -1) {
                TableColumn column = JTableHeader.this.columnModel.getColumn(columnAtPoint);
                TableCellRenderer tableCellRenderer = column.getHeaderRenderer();
                if (tableCellRenderer == null) {
                    if (JTableHeader.this.defaultRenderer != null) {
                        tableCellRenderer = JTableHeader.this.defaultRenderer;
                    }
                    else {
                        return null;
                    }
                }
                tableCellRenderer.getTableCellRendererComponent(JTableHeader.this.getTable(), column.getHeaderValue(), false, false, -1, columnAtPoint);
                return new JTableHeader.AccessibleJTableHeader.AccessibleJTableHeaderEntry(columnAtPoint, JTableHeader.this, JTableHeader.this.table);
            }
            else {
                return null;
            }
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            return JTableHeader.this.columnModel.getColumnCount();
        }
        
        @Override
        public Accessible getAccessibleChild(int n) {
            if (n < 0 || n >= this.getAccessibleChildrenCount()) {
                return null;
            }
            TableColumn column = JTableHeader.this.columnModel.getColumn(n);
            TableCellRenderer tableCellRenderer = column.getHeaderRenderer();
            if (tableCellRenderer == null) {
                if (JTableHeader.this.defaultRenderer != null) {
                    tableCellRenderer = JTableHeader.this.defaultRenderer;
                }
                else {
                    return null;
                }
            }
            tableCellRenderer.getTableCellRendererComponent(JTableHeader.this.getTable(), column.getHeaderValue(), false, false, -1, n);
            return new JTableHeader.AccessibleJTableHeader.AccessibleJTableHeaderEntry(n, JTableHeader.this, JTableHeader.this.table);
        }
        
        protected class AccessibleJTableHeaderEntry extends AccessibleContext implements Accessible, AccessibleComponent
        {
            private JTableHeader parent;
            private int column;
            private JTable table;
            
            public AccessibleJTableHeaderEntry(int column, JTableHeader parent, JTable table) {
                this.parent = parent;
                this.column = column;
                this.table = table;
                this.setAccessibleParent(this.parent);
            }
            
            @Override
            public AccessibleContext getAccessibleContext() {
                return this;
            }
            
            private AccessibleContext getCurrentAccessibleContext() {
                TableColumnModel columnModel = this.table.getColumnModel();
                if (columnModel != null) {
                    if (this.column < 0 || this.column >= columnModel.getColumnCount()) {
                        return null;
                    }
                    TableColumn column = columnModel.getColumn(this.column);
                    TableCellRenderer tableCellRenderer = column.getHeaderRenderer();
                    if (tableCellRenderer == null) {
                        if (JTableHeader.this.defaultRenderer != null) {
                            tableCellRenderer = JTableHeader.this.defaultRenderer;
                        }
                        else {
                            return null;
                        }
                    }
                    Component tableCellRendererComponent = tableCellRenderer.getTableCellRendererComponent(JTableHeader.this.getTable(), column.getHeaderValue(), false, false, -1, this.column);
                    if ((tableCellRendererComponent instanceof Accessible)) {
                        return ((Accessible)tableCellRendererComponent).getAccessibleContext();
                    }
                }
                return null;
            }
            
            private Component getCurrentComponent() {
                TableColumnModel columnModel = this.table.getColumnModel();
                if (columnModel != null) {
                    if (this.column < 0 || this.column >= columnModel.getColumnCount()) {
                        return null;
                    }
                    TableColumn column = columnModel.getColumn(this.column);
                    TableCellRenderer tableCellRenderer = column.getHeaderRenderer();
                    if (tableCellRenderer == null) {
                        if (JTableHeader.this.defaultRenderer != null) {
                            tableCellRenderer = JTableHeader.this.defaultRenderer;
                        }
                        else {
                            return null;
                        }
                    }
                    return tableCellRenderer.getTableCellRendererComponent(JTableHeader.this.getTable(), column.getHeaderValue(), false, false, -1, this.column);
                }
                else {
                    return null;
                }
            }
            
            @Override
            public String getAccessibleName() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    String accessibleName = currentAccessibleContext.getAccessibleName();
                    if (accessibleName != null && accessibleName != "") {
                        return accessibleName;
                    }
                }
                if (this.accessibleName != null && this.accessibleName != "") {
                    return this.accessibleName;
                }
                String s = (String)JTableHeader.this.getClientProperty("AccessibleName");
                if (s != null) {
                    return s;
                }
                return this.table.getColumnName(this.column);
            }
            
            @Override
            public void setAccessibleName(String s) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.setAccessibleName(s);
                }
                else {
                    super.setAccessibleName(s);
                }
            }
            
            @Override
            public String getAccessibleDescription() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleDescription();
                }
                return super.getAccessibleDescription();
            }
            
            @Override
            public void setAccessibleDescription(String s) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.setAccessibleDescription(s);
                }
                else {
                    super.setAccessibleDescription(s);
                }
            }
            
            @Override
            public AccessibleRole getAccessibleRole() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleRole();
                }
                return AccessibleRole.COLUMN_HEADER;
            }
            
            @Override
            public AccessibleStateSet getAccessibleStateSet() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    AccessibleStateSet accessibleStateSet = currentAccessibleContext.getAccessibleStateSet();
                    if (this.isShowing()) {
                        accessibleStateSet.add(AccessibleState.SHOWING);
                        return accessibleStateSet;
                    }
                    return accessibleStateSet;
                }
                else {
                    return new AccessibleStateSet();
                }
            }
            
            @Override
            public int getAccessibleIndexInParent() {
                return this.column;
            }
            
            @Override
            public int getAccessibleChildrenCount() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleChildrenCount();
                }
                return 0;
            }
            
            @Override
            public Accessible getAccessibleChild(int n) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    Accessible accessibleChild = currentAccessibleContext.getAccessibleChild(n);
                    currentAccessibleContext.setAccessibleParent(this);
                    return accessibleChild;
                }
                return null;
            }
            
            @Override
            public Locale getLocale() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getLocale();
                }
                return null;
            }
            
            @Override
            public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.addPropertyChangeListener(propertyChangeListener);
                }
                else {
                    super.addPropertyChangeListener(propertyChangeListener);
                }
            }
            
            @Override
            public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.removePropertyChangeListener(propertyChangeListener);
                }
                else {
                    super.removePropertyChangeListener(propertyChangeListener);
                }
            }
            
            @Override
            public AccessibleAction getAccessibleAction() {
                return this.getCurrentAccessibleContext().getAccessibleAction();
            }
            
            @Override
            public AccessibleComponent getAccessibleComponent() {
                return this;
            }
            
            @Override
            public AccessibleSelection getAccessibleSelection() {
                return this.getCurrentAccessibleContext().getAccessibleSelection();
            }
            
            @Override
            public AccessibleText getAccessibleText() {
                return this.getCurrentAccessibleContext().getAccessibleText();
            }
            
            @Override
            public AccessibleValue getAccessibleValue() {
                return this.getCurrentAccessibleContext().getAccessibleValue();
            }
            
            @Override
            public Color getBackground() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getBackground();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getBackground();
                }
                return null;
            }
            
            @Override
            public void setBackground(Color color) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setBackground(color);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setBackground(color);
                    }
                }
            }
            
            @Override
            public Color getForeground() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getForeground();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getForeground();
                }
                return null;
            }
            
            @Override
            public void setForeground(Color color) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setForeground(color);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setForeground(color);
                    }
                }
            }
            
            @Override
            public Cursor getCursor() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getCursor();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getCursor();
                }
                Accessible accessibleParent = this.getAccessibleParent();
                if ((accessibleParent instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)accessibleParent).getCursor();
                }
                return null;
            }
            
            @Override
            public void setCursor(Cursor cursor) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setCursor(cursor);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setCursor(cursor);
                    }
                }
            }
            
            @Override
            public Font getFont() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getFont();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getFont();
                }
                return null;
            }
            
            @Override
            public void setFont(Font font) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setFont(font);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setFont(font);
                    }
                }
            }
            
            @Override
            public FontMetrics getFontMetrics(Font font) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getFontMetrics(font);
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getFontMetrics(font);
                }
                return null;
            }
            
            @Override
            public boolean isEnabled() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).isEnabled();
                }
                Component currentComponent = this.getCurrentComponent();
                return currentComponent != null && currentComponent.isEnabled();
            }
            
            @Override
            public void setEnabled(boolean b) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setEnabled(b);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setEnabled(b);
                    }
                }
            }
            
            @Override
            public boolean isVisible() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).isVisible();
                }
                Component currentComponent = this.getCurrentComponent();
                return currentComponent != null && currentComponent.isVisible();
            }
            
            @Override
            public void setVisible(boolean b) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setVisible(b);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setVisible(b);
                    }
                }
            }
            
            @Override
            public boolean isShowing() {
                return this.isVisible() && JTableHeader.this.isShowing();
            }
            
            @Override
            public boolean contains(Point p) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getBounds().contains(p);
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getBounds().contains(p);
                }
                return this.getBounds().contains(p);
            }
            
            @Override
            public Point getLocationOnScreen() {
                if (this.parent != null) {
                    Point locationOnScreen = this.parent.getLocationOnScreen();
                    Point location = this.getLocation();
                    location.translate(locationOnScreen.x, locationOnScreen.y);
                    return location;
                }
                return null;
            }
            
            @Override
            public Point getLocation() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getBounds().getLocation();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getBounds().getLocation();
                }
                return this.getBounds().getLocation();
            }
            
            @Override
            public void setLocation(Point point) {
            }
            
            @Override
            public Rectangle getBounds() {
                Rectangle cellRect = this.table.getCellRect(-1, this.column, false);
                cellRect.y = 0;
                return cellRect;
            }
            
            @Override
            public void setBounds(Rectangle rectangle) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setBounds(rectangle);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setBounds(rectangle);
                    }
                }
            }
            
            @Override
            public Dimension getSize() {
                return this.getBounds().getSize();
            }
            
            @Override
            public void setSize(Dimension dimension) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setSize(dimension);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setSize(dimension);
                    }
                }
            }
            
            @Override
            public Accessible getAccessibleAt(Point point) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getAccessibleAt(point);
                }
                return null;
            }
            
            @Override
            public boolean isFocusTraversable() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).isFocusTraversable();
                }
                Component currentComponent = this.getCurrentComponent();
                return currentComponent != null && currentComponent.isFocusTraversable();
            }
            
            @Override
            public void requestFocus() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).requestFocus();
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.requestFocus();
                    }
                }
            }
            
            @Override
            public void addFocusListener(FocusListener l) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).addFocusListener(l);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.addFocusListener(l);
                    }
                }
            }
            
            @Override
            public void removeFocusListener(FocusListener l) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).removeFocusListener(l);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.removeFocusListener(l);
                    }
                }
            }
        }
    }
}
